"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _digitalOcean = _interopRequireDefault(require("./digital-ocean"));

var _utils = require("./utils");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function createSourceConfig(SourceAPI) {
  return {
    async load({
      name,
      groupConfig
    }, pluginApi) {
      const api = new SourceAPI(name, groupConfig, pluginApi);
      return api.getServers();
    },

    async upToDate({
      name,
      groupConfig,
      list
    }, pluginApi) {
      const api = new SourceAPI(name, groupConfig, pluginApi);
      const {
        wrong,
        good,
        toResize = []
      } = await api.compareServers(list);
      return wrong.length === 0 && good.length === groupConfig.count && toResize.length === 0;
    },

    async update({
      name,
      groupConfig
    }, pluginApi) {
      const api = new SourceAPI(name, groupConfig, pluginApi);
      const {
        wrong,
        good,
        toResize = []
      } = await api.compareServers();
      const addCount = Math.max(0, groupConfig.count - good.length);
      const goodRemoveCount = Math.max(0, good.length - groupConfig.count);

      if (goodRemoveCount > 0) {
        console.log(`=> Removing ${goodRemoveCount} servers for ${name}`);
        await api.removeServers(good.slice(0, goodRemoveCount));
        console.log(`=> Finished removing ${goodRemoveCount} servers for ${name}`);
      }

      if (wrong.length > 0) {
        // TODO: don't delete tempCount until mup successfully exits
        console.log(`=> Removing ${wrong.length} servers for ${name}`);
        await api.removeServers(wrong);
        console.log(`=> Finished removing ${wrong.length} servers for ${name}`);
      }

      if (addCount > 0) {
        console.log(`=> Creating ${addCount} servers for ${name}`);
        const created = await api.createServers(addCount);
        await (0, _utils.waitForServers)(created, pluginApi);
        console.log(`=> Finished creating ${addCount} servers for ${name}`);
      }

      for (const server of toResize) {
        console.log(`=> Resizing ${server.__droplet.name} for ${name}`);
        await api.resizeServer(server.__droplet.id);
        await (0, _utils.waitForServers)([server], pluginApi);
        console.log(`=> Finished resizing ${server.__droplet.name} for ${name}`);
      }
    }

  };
}

const serverSources = {
  'digital-ocean': createSourceConfig(_digitalOcean.default)
};
var _default = serverSources;
exports.default = _default;
//# sourceMappingURL=index.js.map