"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.generateName = generateName;
exports.createFingerprint = createFingerprint;
exports.waitForServers = waitForServers;

var _crypto = _interopRequireDefault(require("crypto"));

var _ssh2Classic = require("ssh2-classic");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function generateName(groupName) {
  const randomString = _crypto.default.randomBytes(4).toString('hex');

  return `${groupName}-${randomString}`;
}

function createFingerprint(keyContent) {
  const cleanedContent = keyContent // Remove key type at beginning
  .replace(/(^ssh-[a-zA-Z0-9]*)/, '').trim() // Remove comment at end
  .replace(/ [^ ]+$/, '');
  const buffer = Buffer.from(cleanedContent, 'base64');

  const hash = _crypto.default.createHash('md5').update(buffer).digest('hex'); // Add colons between every 2 characters


  return hash.match(/.{1,2}/g).join(':');
}

const FIVE_MINUTES = 1000 * 60 * 5;

function waitForServers(servers, api) {
  async function waitForServer(server, startTime = Date.now()) {
    if (Date.now() - startTime > FIVE_MINUTES) {
      throw new Error('Timed out waiting for server to accept SSH connections');
    }

    try {
      await new Promise((resolve, reject) => {
        const ssh = api._createSSHOptions(server);

        const conn = new _ssh2Classic.Client();
        conn.once('error', err => {
          reject(err);
        }).once('ready', () => {
          conn.end();
          resolve();
        }).connect(ssh);
      });
    } catch (e) {
      if (e.code !== 'ECONNREFUSED') {
        console.dir(e);
      }

      await new Promise(resolve => setTimeout(resolve, 1000 * 5));
      return waitForServer(server);
    }
  }

  return Promise.all(servers.map(server => waitForServer(server)));
}
//# sourceMappingURL=utils.js.map