"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.gracefulShutdown = gracefulShutdown;
exports.readdInstance = readdInstance;

var _commandHandlers = require("./command-handlers");

var _utils = require("./utils");

function runScript(sessions, script, vars) {
  const promises = sessions.map(session => new Promise((resolve, reject) => {
    session.executeScript(script, {
      vars
    }, (err, code, output) => {
      console.log(err, code, output);

      if (code > 0) {
        return reject(output);
      }

      resolve();
    });
  }));
  return Promise.all(promises);
}

function updateUpstreams(api, toDrain) {
  const {
    app: appConfig,
    proxy: config
  } = api.getConfig();
  const hostnames = (0, _utils.getLoadBalancingHosts)(api.expandServers(appConfig.servers), toDrain ? [toDrain] : undefined);
  const domains = config.domains.split(',');
  const proxySessions = (0, _utils.getSessions)(api); // TODO: we don't need to update the domains

  return runScript(proxySessions, api.resolvePath(__dirname, 'assets/upstream.sh'), {
    domains,
    name: appConfig.name,
    setUpstream: !api.swarmEnabled() && config.loadBalancing,
    stickySessions: config.stickySessions !== false,
    proxyName: _commandHandlers.PROXY_CONTAINER_NAME,
    port: appConfig.env.PORT,
    hostnames
  });
}

async function gracefulShutdown(api, {
  session
}) {
  console.log('graceful shutdown');
  const {
    proxy
  } = api.getConfig();

  if (!proxy || !proxy.loadBalancing) {
    return;
  }

  await updateUpstreams(api, session._host);
}

async function readdInstance(api) {
  const {
    proxy
  } = api.getConfig();

  if (!proxy || !proxy.loadBalancing) {
    return;
  }

  await updateUpstreams(api);
}
//# sourceMappingURL=graceful-shutdown.js.map